==> python_redux-0.9.6.tar.gz: python_redux-0.9.6/redux/combine_reducers.py <==
# ruff: noqa: A003, D100, D101, D102, D103, D104, D105, D107
from __future__ import annotations

import copy
import uuid
from dataclasses import asdict, make_dataclass
from typing import TYPE_CHECKING, Any, TypeGuard, TypeVar, cast

from .basic_types import (
    Action,
    BaseAction,
    BaseEvent,
    CompleteReducerResult,
    Event,
    Immutable,
    InitAction,
    is_reducer_result,
)

if TYPE_CHECKING:
    from redux import ReducerType


class BaseCombineReducerState(Immutable):
    _id: str


class BaseCombineReducerAction(BaseAction):
    _id: str


class CombineReducerRegisterAction(BaseCombineReducerAction):
    key: str
    reducer: ReducerType


class CombineReducerUnregisterAction(BaseCombineReducerAction):
    key: str


CombineReducerAction = CombineReducerRegisterAction | CombineReducerUnregisterAction
CombineReducerState = TypeVar(
    'CombineReducerState',
    bound=BaseCombineReducerState,
)
AnyAction = TypeVar('AnyAction', bound=BaseAction)


def is_combine_reducer_action(action: BaseAction) -> TypeGuard[CombineReducerAction]:
    return isinstance(action, BaseCombineReducerAction)


def combine_reducers(
    state_type: type[CombineReducerState],
    action_type: type[Action] = BaseAction,  # noqa: ARG001
    event_type: type[Event] = BaseEvent,  # noqa: ARG001
    **reducers: ReducerType,
) -> tuple[ReducerType[CombineReducerState, Action, Event], str]:
    _id = uuid.uuid4().hex

    state_class = cast(
        type[state_type],
        make_dataclass(
            'combined_reducer',
            ('_id', *reducers.keys()),
            frozen=True,
            kw_only=True,
        ),
    )

    def combined_reducer(
        state: CombineReducerState | None,
        action: Action,
    ) -> CompleteReducerResult[CombineReducerState, Action, Event]:
        nonlocal state_class
        if state is not None and is_combine_reducer_action(action):
            if isinstance(action, CombineReducerRegisterAction) and action._id == _id:  # noqa: SLF001
                key = action.key
                reducer = action.reducer
                reducers[key] = reducer
                state_class = make_dataclass(
                    'combined_reducer',
                    ('_id', *reducers.keys()),
                    frozen=True,
                )
                state = state_class(
                    _id=state._id,  # noqa: SLF001
                    **(
                        {
                            key_: reducer(None, InitAction())
                            if key == key_
                            else getattr(state, key_)
                            for key_ in reducers
                        }
                    ),
                )
            elif (
                isinstance(action, CombineReducerUnregisterAction) and action._id == _id  # noqa: SLF001
            ):
                key = action.key

                del reducers[key]
                fields_copy = copy.copy(cast(Any, state_class).__dataclass_fields__)
                annotations_copy = copy.deepcopy(state_class.__annotations__)
                del fields_copy[key]
                del annotations_copy[key]
                state_class = make_dataclass('combined_reducer', annotations_copy)
                cast(Any, state_class).__dataclass_fields__ = fields_copy

                state = state_class(
                    **{
                        key_: getattr(state, key_)
                        for key_ in asdict(state)
                        if key_ != key
                    },
                )

        reducers_results = {
            key: reducer(
                None if state is None else getattr(state, key),
                action,
            )
            for key, reducer in reducers.items()
        }
        result_state = state_class(
            _id=_id,
            **{
                key: result.state if is_reducer_result(result) else result
                for key, result in reducers_results.items()
            },
        )
        result_actions = sum(
            [
                result.actions or [] if is_reducer_result(result) else []
                for result in reducers_results.values()
            ],
            [],
        )
        result_events = sum(
            [
                result.events or [] if is_reducer_result(result) else []
                for result in reducers_results.values()
            ],
            [],
        )

        return CompleteReducerResult(
            state=result_state,
            actions=result_actions,
            events=result_events,
        )

    return (combined_reducer, _id)

==> python_redux-0.9.23.tar.gz: python_redux-0.9.23/redux/combine_reducers.py <==
# ruff: noqa: A003, D100, D101, D102, D103, D104, D105, D107
from __future__ import annotations

import copy
import uuid
from dataclasses import asdict, make_dataclass
from typing import TYPE_CHECKING, Any, TypeVar, cast

from .basic_types import (
    Action,
    BaseAction,
    BaseEvent,
    CompleteReducerResult,
    Event,
    Immutable,
    InitAction,
    is_reducer_result,
)

if TYPE_CHECKING:
    from redux import ReducerType


class BaseCombineReducerState(Immutable):
    _id: str


class CombineReducerAction(BaseAction):
    _id: str


class CombineReducerInitAction(CombineReducerAction, InitAction):
    key: str


class CombineReducerRegisterAction(CombineReducerAction):
    key: str
    reducer: ReducerType


class CombineReducerUnregisterAction(CombineReducerAction):
    key: str


CombineReducerState = TypeVar(
    'CombineReducerState',
    bound=BaseCombineReducerState,
)
AnyAction = TypeVar('AnyAction', bound=BaseAction)


def combine_reducers(
    state_type: type[CombineReducerState],
    action_type: type[Action] = BaseAction,
    event_type: type[Event] = BaseEvent,
    **reducers: ReducerType,
) -> tuple[ReducerType[CombineReducerState, Action, Event], str]:
    _ = action_type, event_type
    reducers = reducers.copy()
    _id = uuid.uuid4().hex

    state_class = cast(
        type[state_type],
        make_dataclass(
            'combined_reducer',
            ('_id', *reducers.keys()),
            frozen=True,
            kw_only=True,
        ),
    )

    def combined_reducer(
        state: CombineReducerState | None,
        action: Action,
    ) -> CompleteReducerResult[CombineReducerState, Action, Event]:
        result_actions = []
        result_events = []
        nonlocal state_class
        if state is not None and isinstance(action, CombineReducerAction):
            if isinstance(action, CombineReducerRegisterAction) and action._id == _id:  # noqa: SLF001
                key = action.key
                reducer = action.reducer
                reducers[key] = reducer
                state_class = make_dataclass(
                    'combined_reducer',
                    ('_id', *reducers.keys()),
                    frozen=True,
                )
                reducer_result = reducer(
                    None,
                    CombineReducerInitAction(_id=_id, key=key),
                )
                state = state_class(
                    _id=state._id,  # noqa: SLF001
                    **(
                        {
                            key_: (
                                reducer_result.state
                                if is_reducer_result(reducer_result)
                                else reducer_result
                            )
                            if key == key_
                            else getattr(state, key_)
                            for key_ in reducers
                        }
                    ),
                )
                result_actions += (
                    reducer_result.actions or []
                    if is_reducer_result(reducer_result)
                    else []
                )
                result_events += (
                    reducer_result.events or []
                    if is_reducer_result(reducer_result)
                    else []
                )
            elif (
                isinstance(action, CombineReducerUnregisterAction) and action._id == _id  # noqa: SLF001
            ):
                key = action.key

                del reducers[key]
                fields_copy = copy.copy(cast(Any, state_class).__dataclass_fields__)
                annotations_copy = copy.deepcopy(state_class.__annotations__)
                del fields_copy[key]
                del annotations_copy[key]
                state_class = make_dataclass('combined_reducer', annotations_copy)
                cast(Any, state_class).__dataclass_fields__ = fields_copy

                state = state_class(
                    **{
                        key_: getattr(state, key_)
                        for key_ in asdict(state)
                        if key_ != key
                    },
                )

        reducers_results = {
            key: reducer(
                None if state is None else getattr(state, key),
                CombineReducerInitAction(key=key, _id=_id)
                if isinstance(action, InitAction)
                else action,
            )
            for key, reducer in reducers.items()
        }
        result_state = state_class(
            _id=_id,
            **{
                key: result.state if is_reducer_result(result) else result
                for key, result in reducers_results.items()
            },
        )
        result_actions += sum(
            [
                result.actions or [] if is_reducer_result(result) else []
                for result in reducers_results.values()
            ],
            [],
        )
        result_events += sum(
            [
                result.events or [] if is_reducer_result(result) else []
                for result in reducers_results.values()
            ],
            [],
        )

        return CompleteReducerResult(
            state=result_state,
            actions=result_actions,
            events=result_events,
        )

    return combined_reducer, _id
